# ファイル名
FNAME_MAPPING = '_mapping.yaml'

# レイアウト
LAYOUT_TABLE = 'table'
LAYOUT_MATRIX = 'matrix'
LAYOUT_SINGLE = 'single'

# カラム定義
BIND_TYPE_FIXED = 'fixed'
BIND_TYPE_CALL = 'call'
BIND_TYPE_EXTEND = 'extend'
# カラム定義（特殊タイプ）
BIND_TYPE_REFS = 'refs'
BIND_TYPE_BLANK = ''

# callのタイプ
CALL_TYPE_UUID = 'uuid'
CALL_TYPE_ULID = 'ulid'

# FIXME セルタイプ
FIELD_TYPE_STRING = 'string'
FIELD_TYPE_SELECTABLE = 'singleSelect'
FIELD_TYPE_SYSTEM = 'system'

# 同期モード
SYNC_MODE_DROP_CREATE = 'drop_create'

# フィールド幅
DEFAULT_WIDTH = 150
